@@ -101,6 +101,11 @@ import Module
 import qualified Data.IntSet as I (IntSet, empty)
 #endif
 
+#if __GLASGOW_HASKELL__ < 706
+import Control.DeepSeq (NFData(rnf))
+import Data.ByteString.Lazy.Internal (ByteString(..))
+#endif
+
 import Bag
 import Lexer as L
 import Parser
@@ -370,7 +375,7 @@ pprInfo m pefas (thing, fixity, insts)
                                         (char '\t' <> ptext (sLit "--") <+> loc)
                          where loc = ptext (sLit "Defined") <+> pprNameDefnLoc' (getName thing')
 #else
-    pprTyThingInContextLoc' pefas thing' = hang (pprTyThingInContext pefas thing') 2
+    pprTyThingInContextLoc' pefas' thing' = hang (pprTyThingInContext pefas' thing') 2
                                     (char '\t' <> ptext (sLit "--") <+> loc)
                                where loc = ptext (sLit "Defined") <+> pprNameDefnLoc' (getName thing')
 #endif
@@ -564,3 +569,9 @@ mkErrStyle' = Outputable.mkErrStyle
 #else
 mkErrStyle' _ = Outputable.mkErrStyle
 #endif
+
+#if __GLASGOW_HASKELL__ < 706
+instance NFData ByteString where
+  rnf Empty       = ()
+  rnf (Chunk _ b) = rnf b
+#endif
