@@ -4,8 +4,8 @@ import Exception (ghandle)
 import Control.Exception (SomeException(..))
 import Language.Haskell.GhcMod.Logger (checkErrorPrefix)
 import Language.Haskell.GhcMod.Convert
-import Language.Haskell.GhcMod.Monad
 import Language.Haskell.GhcMod.Types
+import Language.Haskell.GhcMod.Monad
 import Language.Haskell.HLint (hlint)
 
 import Language.Haskell.GhcMod.Utils (withMappedFile)
@@ -15,12 +15,12 @@ import Data.List (stripPrefix)
 -- | Checking syntax of a target file using hlint.
 --   Warnings and errors are returned.
 lint :: IOish m
-     => FilePath  -- ^ A target file.
+     => LintOpts  -- ^ Configuration parameters
+     -> FilePath  -- ^ A target file.
      -> GhcModT m String
-lint file = do
-  opt <- options
+lint opt file =
   withMappedFile file $ \tempfile ->
-        liftIO (hlint $ tempfile : "--quiet" : optHlintOpts opt)
+        liftIO (hlint $ tempfile : "--quiet" : optLintHlintOpts opt)
     >>= mapM (replaceFileName tempfile)
     >>= ghandle handler . pack
  where
